<?php

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2011 Manfred Egger <em@bluechip.at>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *  A copy is found in the textfile GPL.txt and important notices to the license
 *  from the author is found in LICENSE.txt distributed with these scripts.
 *
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 * Some common methods to make some things easier
 *
 * @author	Manfred Egger <em@bluechip.at>
 */
class tx_bc_post2facebook_div {
	

	/**
	 * StartsWith
	 * Tests if a text starts with an given string.
	 *
	 * @param     string
	 * @param     string
	 * @return    bool
	 */
	static function StartsWith($Haystack, $Needle) {
		// Recommended version, using strpos
		return strpos($Haystack, $Needle) === 0;
	}

	/**
	 * EndsWith
	 * Tests whether a text ends with the given
	 * string or not.
	 *
	 * @param     string
	 * @param     string
	 * @return    bool
	 */
	static function EndsWith($Haystack, $Needle) {
		// Recommended version, using strpos
		return strrpos($Haystack, $Needle) === strlen($Haystack) - strlen($Needle);
	}

	static function message($message, $header, $type) {
		return '
			<div class="typo3-message '.$type.'">
				<div class="message-header">' . $header . '</div>
				<div class="message-body">
					' . $message . '
				</div>
			</div>';
	}

	static function getRelativePath( $path, $compareTo = '' ) {
		if(!$compareTo) {
			$compareTo = t3lib_div::getIndpEnv('TYPO3_DOCUMENT_ROOT').dirname($_SERVER['PHP_SELF']);
		}
        // clean arguments by removing trailing and prefixing slashes
        if ( substr( $path, -1 ) == '/' ) {
            $path = substr( $path, 0, -1 );
        }
        if ( substr( $path, 0, 1 ) == '/' ) {
            $path = substr( $path, 1 );
        }

        if ( substr( $compareTo, -1 ) == '/' ) {
            $compareTo = substr( $compareTo, 0, -1 );
        }
        if ( substr( $compareTo, 0, 1 ) == '/' ) {
            $compareTo = substr( $compareTo, 1 );
        }

        // simple case: $compareTo is in $path
        if ( strpos( $path, $compareTo ) === 0 ) {
            $offset = strlen( $compareTo ) + 1;
            return substr( $path, $offset );
        }

        $relative  = array(  );
        $pathParts = explode( '/', $path );
        $compareToParts = explode( '/', $compareTo );

        foreach( $compareToParts as $index => $part ) {
            if ( isset( $pathParts[$index] ) && $pathParts[$index] == $part ) {
                continue;
            }

            $relative[] = '..';
        }

        foreach( $pathParts as $index => $part ) {
            if ( isset( $compareToParts[$index] ) && $compareToParts[$index] == $part ) {
                continue;
            }

            $relative[] = $part;
        }

        return implode( '/', $relative );
    }

	function getConfig($pageUid) {
		$fbAppSettings = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['bc_post2facebook']);
		$conf = $this->loadTS($pageUid);

		if(!is_array($fbAppSettings)) $fbAppSettings = array();
		if(!is_array($conf)) $conf = array();

		$fbAppSettings = t3lib_div::array_merge_recursive_overrule($fbAppSettings, $conf, false);
		$fbAppSettings['webUrl'] = $this->normalizeWebUrl($fbAppSettings['webUrl']);
		return $fbAppSettings;
	}

	function normalizeWebUrl($url) {
		if (!$url) {
			$url = $this->getWebUrl();
		}
		$parsed = parse_url($url);
		if(!$parsed['scheme']) {
			$url = 'http://'.$url;
		}
		if(!tx_bc_post2facebook_div::EndsWith($url,'/')) {
			$url .= '/';
		}
		return $url;
	}

	function loadTS($pageUid) {
		if($GLOBALS['BE_USER']) {
			$temp = t3lib_BEfunc::getModTSconfig($pageUid,'mod.web_modules.bc_post2facebook');
			return $temp['properties'];
		} else {
			if($TSFE->sys_page) {
				$sysPageObj = $TSFE->sys_page;
			} else {
				$sysPageObj = t3lib_div::makeInstance('t3lib_pageSelect');
			}
			$TSObj = t3lib_div::makeInstance('t3lib_tsparser_ext');
			$TSObj->tt_track = 0;
			$TSObj->init();
			$rootLine = $sysPageObj->getRootLine($id);
			$TSObj->runThroughTemplates($rootLine);
			$TSObj->generateConfig();
			return $TSObj->setup['plugin.']['tx_bcpost2facebook.'];
		}
	}

	function getWebUrl() {
		if ($this->TSObj && $this->TSObj->setup['baseURL']) {
			return $this->TSObj->setup['baseURL'];
		} else {
			return t3lib_div::getIndpEnv('TYPO3_SITE_URL');
		}
	}

	function initTSFE($id) {

		$GLOBALS['TT'] = t3lib_div::makeInstance('t3lib_timeTrack');
		$GLOBALS['TT']->start();
		
		$TSFE = t3lib_div::makeInstance('tslib_fe',
			$GLOBALS['TYPO3_CONF_VARS'],
			t3lib_div::_GP('id'),
			t3lib_div::_GP('type'),
			t3lib_div::_GP('no_cache'),
			t3lib_div::_GP('cHash'),
			t3lib_div::_GP('jumpurl'),
			t3lib_div::_GP('MP'),
			t3lib_div::_GP('RDCT')
		);

		$TSFE->id = $id;
		$TSFE->connectToDB();
		$TSFE->initFEuser();
		$TSFE->sys_page = t3lib_div::makeInstance('t3lib_pageSelect');
		$TSFE->sys_page->init($TSFE->showHiddenPage);

		$pParts = explode('.',$TSFE->id);
		$TSFE->id = $pParts[0];	// Set it.
		if (isset($pParts[1]))	{$TSFE->type=$pParts[1];}
		$TSFE->idParts = explode(',',$TSFE->id);
		list($pgID,$SSD_p)=explode('+',$TSFE->idParts[0],2);
		if ($SSD_p)	{ $TSFE->idPartsAnalyze($SSD_p); }
		$TSFE->id = $pgID;
		$TSFE->checkAndSetAlias();
		$TSFE->id = intval($TSFE->id);
		$TSFE->type = intval($TSFE->type);

		$TSFE->getPageAndRootline();
		$TSFE->initTemplate();
		$TSFE->forceTemplateParsing = 1;
		$TSFE->getConfigArray();
		$TSFE->initUserGroups();
		$TSFE->initTemplate();
		$TSFE->connectToDB();
		$TSFE->tmpl->getFileName_backPath=PATH_site;
		$GLOBALS['TSFE'] = $TSFE;
		$GLOBALS['LANG']->includeLLFile('EXT:bc_post2facebook/locallang_db.xml');
	}
}

// Include extension?
if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/bc_post2facebook/class.tx_bc_post2facebook_div.php']) {
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/bc_post2facebook/lib/class.tx_bc_post2facebook_div.php']);
}
?>