<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2011 Manfred Egger <em@bluechip.at>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *  A copy is found in the textfile GPL.txt and important notices to the license
 *  from the author is found in LICENSE.txt distributed with these scripts.
 *
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/


require_once(t3lib_extMgm::extPath('bc_post2facebook') . 'lib/class.tx_bc_post2facebook_div.php');
require_once(t3lib_extMgm::extPath('bc_post2facebook') . 'lib/facebook/class.tx_bc_post2facebook_facebook.php');

class tx_bc_post2facebook_tsparserext {


	/**
	 * [Describe function...]
	 *
	 * @return	[type]		...
	 */
	function displayMessage(&$params, &$tsObj) {
		$out = '';

		if (t3lib_div::int_from_ver(TYPO3_version) < 4003000) {
			// 4.3.0 comes with flashmessages styles. For older versions we include the needed styles here.
			$cssPath = $GLOBALS['BACK_PATH'] . t3lib_extMgm::extRelPath('bc_post2facebook');
			$out .= '<link rel="stylesheet" type="text/css" href="' . $cssPath . 'compat/flashmessages.css" media="screen" />';
		}

		if (!function_exists('curl_init')) {
			$errorText[] = $GLOBALS['LANG']->sL('LLL:EXT:bc_post2facebook/locallang_db.xml:extConf.noCurl');
		}
		if (!function_exists('json_decode')) {
			$errorText[] = $GLOBALS['LANG']->sL('LLL:EXT:bc_post2facebook/locallang_db.xml:extConf.noJson');
		}

		$this->confArr = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['bc_post2facebook']);
		$post = t3lib_div::_POST();
		if (count($post) > 0) {
			$this->confArr = array_merge($this->confArr,$post['data']);
		}

		$permissions = t3lib_div::trimExplode(',', $this->confArr['permissions'], true);
		
		if(count($errorText)) {
			$out = $this->createMessage(
				implode('<br />',$errorText),
				'error'
			);
		} else {
			if($this->confArr['appId'] && $this->confArr['secret'] && count($permissions)>1) {
				$div = t3lib_div::makeInstance('tx_bc_post2facebook_div');
				$fbAppSettings = $this->confArr;
				$fbAppSettings['appUrl'] = $div->normalizeWebUrl($fbAppSettings['appUrl']);
				$this->bcFacebook = t3lib_div::makeInstance('tx_bc_post2facebook_facebook');
				$this->bcFacebook->initFacebook($fbAppSettings);
				try {
					$authInfo = $this->isAuthTokenInvalid($permissions);
				} catch(BC_FacebookApiException $ex) {
					$out = $this->createMessage(
						$GLOBALS['LANG']->sL('LLL:EXT:bc_post2facebook/locallang_db.xml:extConf.cUrlProblem').'<br /><br /><i>'.nl2br($ex).'</i>',
						'error'
					);
				}
				$newToken = '
					<br /><br /><a target="_blank" style="text-decoration:underline;" href="'.
					$this->bcFacebook->getLoginUrl($permissions).
					'">'.$GLOBALS['LANG']->sL('LLL:EXT:bc_post2facebook/locallang_db.xml:extConf.newToken').'</a>';

				if(!$out) {
					if(!$authInfo) {
						$out = $this->createMessage(
							$GLOBALS['LANG']->sL('LLL:EXT:bc_post2facebook/locallang_db.xml:extConf.tokenOk'),
							'information'
						);
					} elseif($authInfo == 'noPermission') {
						$out = $this->createMessage(
							$GLOBALS['LANG']->sL('LLL:EXT:bc_post2facebook/locallang_db.xml:extConf.tokenPermissions').$newToken,
							'warning'
						);
					} elseif($authInfo == 'noSession' || $authInfo == 'noToken') {
						$out = $this->createMessage(
							$GLOBALS['LANG']->sL('LLL:EXT:bc_post2facebook/locallang_db.xml:extConf.tokenInvalid').$newToken,
							'error'
						);
					}
				}

				// Check, if Url is Ok
				if($out && $fbAppSettings['appUrl'] !== $this->confArr['appUrl']) {
					$out .= $this->createMessage(
						sprintf(
							$GLOBALS['LANG']->sL('LLL:EXT:bc_post2facebook/locallang_db.xml:extConf.urlInvalid'),
							'<i>'.$fbAppSettings['appUrl'].'</i>'
						),
						'warning'
					);
				}
			}

			if(!$out) {
				$out = $this->createMessage(
					$GLOBALS['LANG']->sL('LLL:EXT:bc_post2facebook/locallang_db.xml:extConf.configInvalid'),
					'error'
				);
			}
		}

		$out = '<div style="position:absolute;top:10px;right:10px; width:300px;">'.$out.'</div>';

		return $out;
	}

	function isAuthTokenInvalid($permissions) {
		if($this->confArr['authToken']) {
			$tokenFound = $this->bcFacebook->get_check_session();
			if($tokenFound) {
				if(!$this->bcFacebook->checkGrantedPermissions($permissions)) {
					return 'noPermission';
				}
			} else {
				return 'noSession';
			}
		} else {
			return 'noToken';
		}
	}

	function createMessage($text, $type) {
		return tx_bc_post2facebook_div::message(
				$text,
				$GLOBALS['LANG']->sL('LLL:EXT:bc_post2facebook/locallang_db.xml:extConf.'.$type.'Header'),
				'message-'.$type
			);
	}
}
if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/bc_post2facebook/lib/class.tx_bc_post2facebook_tsparserext.php']) {
	include_once ($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/bc_post2facebook/lib/class.tx_bc_post2facebook_tsparserext.php']);
}
?>