<?php

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2011 Manfred Egger <em@bluechip.at>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *  A copy is found in the textfile GPL.txt and important notices to the license
 *  from the author is found in LICENSE.txt distributed with these scripts.
 *
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

require_once(t3lib_extMgm::extPath('bc_post2facebook') . 'lib/facebook/class.tx_bc_post2facebook_facebook_graph.php');

class tx_bc_post2facebook_facebook {

	function initFacebook($fbAppSettings) {
		$this->fbAppSettings = $fbAppSettings;
		$params = array(
			'appId' => $this->fbAppSettings['appId'],
			'secret' => $this->fbAppSettings['secret'],
			'cookie' => true,
			'fileUpload' => true,
		);
		$this->facebook = new BcPost2Facebook($params);
	}

	function getLoginUrl(array $params) {
		$urlParams = array(
			'client_id' => $this->fbAppSettings['appId'],
			'redirect_uri' => $this->fbAppSettings['appUrl'],
			'scope' => implode(',', $params),
			'state' => 'bc_post2facebook_'.  md5(uniqid(rand(), TRUE))
		);
		$url = 'https://www.facebook.com/dialog/oauth?' . t3lib_div::implodeArrayForUrl('', $urlParams);
		return $url;
	}

	function checkGrantedPermissions($permissions) {
		$perms = $this->facebook->api('/me/permissions', array(
					'access_token' => $this->token
				));
		foreach ($permissions as $perm) {
			if (!array_key_exists($perm, $perms['data'][0])) {
				return false;
			}
		}
		return true;
	}

	function getTokens($token) {
		$ret = array();
		try {
			$attachment = array('access_token' => $token);
			$accounts = $this->facebook->api('/me/accounts', 'GET', $attachment);
			foreach ($accounts['data'] as $account) {
				$ret[$account['id']] = $account['access_token'];
			}
			$ret['account_info'] = $accounts['data'];
		} catch (Exception $e) {

		}
		try {
			$attachment = array('access_token' => $token);
			$accounts = $this->facebook->api('/me/groups', 'GET', $attachment);
			$ret['group_info'] = $accounts['data'];
		} catch (Exception $e) {

		}
		return $ret;
	}

	function get_check_session() {
		$token = $this->fbAppSettings['authToken'];
		$sessionFile = PATH_site . 'typo3temp/bc_post2facebook_' . sha1($token) . '.txt';
		if (file_exists($sessionFile) && filemtime($sessionFile) > (time() - 60 * 60 * 24 * 60)) {
			$arr = unserialize(t3lib_div::getUrl($sessionFile));
			$this->token = $arr['token'];
			$this->tokens = $arr['tokens'];
			return true;
		} else {
			$splitted = explode('|', $token);
			if (count($splitted) < 3 && strpos($token, '_') > 0) {
				$splitted = t3lib_div::trimExplode('#', $token, true);
				$token = $splitted[0];
				$params = array(
					'client_id' => $this->fbAppSettings['appId'],
					'client_secret' => $this->fbAppSettings['secret'],
					'redirect_uri' => $this->fbAppSettings['appUrl'],
					'code' => $token
				);
				$url = 'https://graph.facebook.com/oauth/access_token?' . t3lib_div::implodeArrayForUrl('', $params);
				$accessToken = self::getURL($url);
				if (!$accessToken)
					return false;
				$urlVars = t3lib_div::explodeUrl2Array($accessToken);
				$token = $urlVars['access_token'];
			}
			if ($token) {
				$this->token = $token;
				$this->facebook->setAccessToken($token);
				$this->facebook->setExtendedAccessToken();
				$this->tokens = $this->getTokens($token);
				
				try {
					$attachment = array('access_token' => $token);
					$this->facebook->api('/me', 'GET', $attachment);
				} catch (Exception $e) {
					return false;
				}
				t3lib_div::writeFile($sessionFile, serialize(array('token' => $token, 'tokens' => $this->tokens)));
				return true;
			}
		}
		return false;
	}

	public static function getUrl($url, $opts = array(), $ch = null) {
		$content = FALSE;

		if (!$ch) {
			if (!function_exists('curl_init') || !($ch = curl_init())) {
				return FALSE;
			}
		}

		if (!is_array($opts) || !count($opts)) {
			$opts = array(
				CURLOPT_CONNECTTIMEOUT => max(0, intval($GLOBALS['TYPO3_CONF_VARS']['SYS']['curlTimeout'])),
				CURLOPT_RETURNTRANSFER => true,
				CURLOPT_TIMEOUT => 60,
				CURLOPT_USERAGENT => 'bc_post2facebook',
			);
		}
		
		if (isset($opts[CURLOPT_HTTPHEADER])) {
			$existing_headers = $opts[CURLOPT_HTTPHEADER];
			$existing_headers[] = 'Expect:';
			$opts[CURLOPT_HTTPHEADER] = $existing_headers;
		} else {
			$opts[CURLOPT_HTTPHEADER] = array('Expect:');
	    }
		
		$opts[CURLOPT_URL] = $url;
		//$opts[CURLOPT_HEADER] = 0;
		//$opts[CURLOPT_NOBODY] = 0;
		$opts[CURLOPT_HTTPGET] = 'GET';
		//$opts[CURLOPT_FOLLOWLOCATION] = 1;

		// (Proxy support implemented by Arco <arco@appeltaart.mine.nu>)
		if ($GLOBALS['TYPO3_CONF_VARS']['SYS']['curlProxyServer']) {
			$opts[CURLOPT_PROXY] = $GLOBALS['TYPO3_CONF_VARS']['SYS']['curlProxyServer'];

			if ($GLOBALS['TYPO3_CONF_VARS']['SYS']['curlProxyTunnel']) {
				$opts[CURLOPT_HTTPPROXYTUNNEL] = $GLOBALS['TYPO3_CONF_VARS']['SYS']['curlProxyTunnel'];
			}
			if ($GLOBALS['TYPO3_CONF_VARS']['SYS']['curlProxyUserPass']) {
				$opts[CURLOPT_PROXYUSERPWD] = $GLOBALS['TYPO3_CONF_VARS']['SYS']['curlProxyUserPass'];
			}
		}
		
		curl_setopt_array($ch, $opts);

		$content = curl_exec($ch);
		$nr = curl_errno($ch);
		if ($nr == 60 || $nr == 77) {
			curl_setopt($ch, CURLOPT_CAPATH, t3lib_extMgm::extPath('bc_post2facebook') . 'lib/facebook/');
			curl_setopt($ch, CURLOPT_CAINFO, t3lib_extMgm::extPath('bc_post2facebook') . 'lib/facebook/fb_ca_chain_bundle.crt');
			$content = curl_exec($ch);
		}
		if ($content === false) {
			throw new BC_FacebookApiException(array(
				'error_code' => curl_errno($ch),
				'error' => array(
					'message' => curl_error($ch),
					'type' => 'CurlException',
				),
			));
		}
		curl_close($ch);
		return $content;
	}

}

// Include extension?
if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/bc_post2facebook/lib/facebook/class.tx_bc_post2facebook_facebook.php']) {
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/bc_post2facebook/lib/facebook/class.tx_bc_post2facebook_facebook.php']);
}
?>